@@ -16,6 +16,11 @@
 #' @param ci_method Method for computing degrees of freedom for p-values
 #'   and confidence intervals (CI). See documentation for related model class
 #'   in [model_parameters()].
+#' @param coefficient_names Character vector with strings that should be used
+#'   as column headers for the coefficient column. Must be of same length as
+#'   number of models in `...`, or length 1. If length 1, this name will be
+#'   used for all coefficient columns. If `NULL`, the name for the coefficient
+#'   column will detected automatically (as in `model_parameters()`).
 #' @inheritParams model_parameters.default
 #' @inheritParams model_parameters.cpglmm
 #' @inheritParams print.parameters_model
@@ -32,7 +37,7 @@
 #'
 #' @return A data frame of indices related to the model's parameters.
 #'
-#' @examples
+#' @examplesIf require("gt", quietly = TRUE)
 #' data(iris)
 #' lm1 <- lm(Sepal.Length ~ Species, data = iris)
 #' lm2 <- lm(Sepal.Length ~ Species + Petal.Length, data = iris)
@@ -41,7 +46,7 @@
 #' # custom style
 #' compare_parameters(lm1, lm2, select = "{estimate}{stars} ({se})")
 #'
-#' \dontrun{
+#' \donttest{
 #' # custom style, in HTML
 #' result <- compare_parameters(lm1, lm2, select = "{estimate}<br>({se})|{p}")
 #' print_html(result)
@@ -51,7 +56,7 @@
 #' m1 <- lm(mpg ~ wt, data = mtcars)
 #' m2 <- glm(vs ~ wt + cyl, data = mtcars, family = "binomial")
 #' compare_parameters(m1, m2)
-#' \dontrun{
+#' \donttest{
 #' # exponentiate coefficients, but not for lm
 #' compare_parameters(m1, m2, exponentiate = "nongaussian")
 #'
@@ -75,8 +80,11 @@ compare_parameters <- function(...,
                                select = NULL,
                                column_names = NULL,
                                pretty_names = TRUE,
+                               coefficient_names = NULL,
                                keep = NULL,
                                drop = NULL,
+                               include_reference = FALSE,
+                               groups = NULL,
                                verbose = TRUE) {
   models <- list(...)
 
@@ -98,26 +106,28 @@ compare_parameters <- function(...,
       names(models) <- model_names
     }
   } else {
-    model_names <- match.call(expand.dots = FALSE)$`...`
+    model_names <- match.call(expand.dots = FALSE)[["..."]]
     if (length(names(model_names)) > 0) {
       model_names <- names(model_names)
     } else if (any(vapply(model_names, is.call, TRUE))) {
       model_names <- paste("Model", seq_along(models), sep = " ")
     } else {
-      model_names <- sapply(model_names, as.character)
+      model_names <- vapply(model_names, as.character, character(1))
       names(models) <- model_names
     }
   }
 
-  supported_models <- sapply(models, function(i) {
-    insight::is_model_supported(i) | inherits(i, "lavaan") | inherits(i, "parameters_model")
-  })
+  supported_models <- vapply(models, function(i) {
+    insight::is_model_supported(i) || inherits(i, "lavaan") || inherits(i, "parameters_model")
+  }, TRUE)
 
   if (!all(supported_models)) {
-    insight::format_warning(
-      sprintf("Following objects are not supported: %s", toString(model_names[!supported_models])),
-      "Dropping unsupported models now."
-    )
+    if (verbose) {
+      insight::format_alert(
+        sprintf("Following objects are not supported: %s", toString(model_names[!supported_models])),
+        "Dropping unsupported models now."
+      )
+    }
     models <- models[supported_models]
     model_names <- model_names[supported_models]
   }
@@ -137,13 +147,18 @@ compare_parameters <- function(...,
   if (!is.null(column_names)) {
     if (length(column_names) != length(model_names)) {
       if (isTRUE(verbose)) {
-        insight::format_warning("Number of column names does not match number of models.")
+        insight::format_alert("Number of column names does not match number of models.")
       }
     } else {
       model_names <- column_names
     }
   }
 
+  # make sure we have enough coefficient names - else, repeat first value
+  if (!is.null(coefficient_names) && length(coefficient_names) < length(models)) {
+    coefficient_names <- rep(coefficient_names[1], length(models))
+  }
+
   # iterate all models and create list of model parameters
   m <- lapply(seq_along(models), function(i) {
     model <- models[[i]]
@@ -154,7 +169,7 @@ compare_parameters <- function(...,
       dat <- model
     } else {
       # set default-ci_type for Bayesian models
-      if (.is_bayesian_model(model) && !ci_method %in% c("hdi", "quantile", "ci", "eti", "si", "bci", "bcai")) {
+      if (.is_bayesian_model(model, exclude = c("bmerMod", "bayesx", "blmerMod", "bglmerMod")) && !ci_method %in% c("hdi", "quantile", "ci", "eti", "si", "bci", "bcai")) { # nolint
         ci_method_tmp <- "eti"
       } else {
         ci_method_tmp <- ci_method
@@ -173,14 +188,17 @@ compare_parameters <- function(...,
         keep = keep,
         drop = drop,
         wb_component = FALSE,
+        include_reference = include_reference,
         verbose = verbose
       )
     }
 
     # set specific names for coefficient column
     coef_name <- attributes(dat)$coefficient_name
-    if (!is.null(coef_name)) {
+    if (!is.null(coef_name) && is.null(coefficient_names)) {
       colnames(dat)[colnames(dat) == "Coefficient"] <- coef_name
+    } else if (!is.null(coefficient_names)) {
+      colnames(dat)[colnames(dat) == "Coefficient"] <- coefficient_names[i]
     }
 
     # set pretty parameter names
@@ -231,13 +249,14 @@ compare_parameters <- function(...,
   all_models[model_cols] <- NULL
 
   # remove empty group-column
-  if (all(nchar(all_models$Group) == 0)) {
+  if (!any(nzchar(as.character(all_models$Group), keepNA = TRUE))) {
     all_models$Group <- NULL
   }
 
   attr(all_models, "model_names") <- gsub("\"", "", unlist(lapply(model_names, insight::safe_deparse)), fixed = TRUE)
   attr(all_models, "output_style") <- select
   attr(all_models, "all_attributes") <- object_attributes
+  attr(all_models, "parameter_groups") <- groups
   class(all_models) <- c("compare_parameters", "see_compare_parameters", unique(class(all_models)))
 
   all_models
@@ -273,14 +292,14 @@ compare_models <- compare_parameters
       }
     } else {
       match_pretty_names <- att$pretty_names[x$Parameter]
-      if (!anyNA(match_pretty_names)) {
-        x$Parameter <- att$pretty_names[x$Parameter]
-      } else {
+      if (anyNA(match_pretty_names)) {
         match_pretty_names <- match(names(att$pretty_names), x$Parameter)
         match_pretty_names <- match_pretty_names[!is.na(match_pretty_names)]
         if (length(match_pretty_names)) {
           x$Parameter[match_pretty_names] <- att$pretty_names[x$Parameter[match_pretty_names]]
         }
+      } else {
+        x$Parameter <- att$pretty_names[x$Parameter]
       }
     }
   }
@@ -291,3 +310,19 @@ compare_models <- compare_parameters
 
   x
 }
+
+
+.is_bayesian_model <- function(x, exclude = NULL) {
+  bayes_classes <- c(
+    "brmsfit", "stanfit", "MCMCglmm", "stanreg",
+    "stanmvreg", "bmerMod", "BFBayesFactor", "bamlss",
+    "bayesx", "mcmc", "bcplm", "bayesQR", "BGGM",
+    "meta_random", "meta_fixed", "meta_bma", "blavaan",
+    "blrm", "blmerMod"
+  )
+  # if exclude is not NULL, remove elements in exclude from bayes_class
+  if (!is.null(exclude)) {
+    bayes_classes <- bayes_classes[!bayes_classes %in% exclude]
+  }
+  inherits(x, bayes_classes)
+}
