@@ -17,15 +17,16 @@ bayestestR::equivalence_test
 #'   equivalence. Can be `"bayes"`, `"classic"` or `"cet"`. See
 #'   'Details'.
 #' @param test Hypothesis test for computing contrasts or pairwise comparisons.
-#'   See [`?ggeffects::hypothesis_test`](https://strengejacke.github.io/ggeffects/reference/hypothesis_test.html)
+#'   See [`?ggeffects::test_predictions`](https://strengejacke.github.io/ggeffects/reference/test_predictions.html)
 #'   for details.
 #' @param verbose Toggle warnings and messages.
 #' @param ... Arguments passed to or from other methods.
 #' @inheritParams model_parameters.merMod
 #' @inheritParams p_value
 #'
-#' @seealso For more details, see [bayestestR::equivalence_test()].
-#'   Further readings can be found in the references.
+#' @seealso For more details, see [bayestestR::equivalence_test()]. Further
+#' readings can be found in the references. See also [`p_significance()`] for
+#' a unidirectional equivalence test.
 #'
 #' @details
 #' In classical null hypothesis significance testing (NHST) within a frequentist
@@ -62,14 +63,17 @@ bayestestR::equivalence_test
 #' - "classic" - The TOST rule (Lakens 2017)
 #'
 #'   This rule follows the "TOST rule", i.e. a two one-sided test procedure
-#'   (_Lakens 2017_). Following this rule, practical equivalence of an effect
-#'   (i.e. H0) is *rejected*, when the coefficient is statistically significant
-#'   *and* the narrow confidence intervals (i.e. `1-2*alpha`) *include* or
-#'   *exceed* the ROPE boundaries. Practical equivalence is assumed
-#'   (i.e. H0 "accepted") when the narrow confidence intervals are completely
-#'   inside the ROPE, no matter if the effect is statistically significant
-#'   or not. Else, the decision whether to accept or reject practical
-#'   equivalence is undecided.
+#'   (_Lakens 2017_). Following this rule...
+#'   - practical equivalence is assumed (i.e. H0 *"accepted"*) when the narrow
+#'     confidence intervals are completely inside the ROPE, no matter if the
+#'     effect is statistically significant or not;
+#'   - practical equivalence (i.e. H0) is *rejected*, when the coefficient is
+#'     statistically significant, both when the narrow confidence intervals
+#'     (i.e. `1-2*alpha`) include or exclude the the ROPE boundaries, but the
+#'     narrow confidence intervals are *not fully covered* by the ROPE;
+#'   - else the decision whether to accept or reject practical equivalence is
+#'     undecided (i.e. when effects are *not* statistically significant *and*
+#'     the narrow confidence intervals overlaps the ROPE).
 #'
 #' - "cet" - Conditional Equivalence Testing (Campbell/Gustafson 2018)
 #'
@@ -100,13 +104,50 @@ bayestestR::equivalence_test
 #' Second generation p-values (SGPV) were proposed as a statistic that
 #' represents _the proportion of data-supported hypotheses that are also null
 #' hypotheses_ _(Blume et al. 2018, Lakens and Delacre 2020)_. It represents the
-#' proportion of the confidence interval range that is inside the ROPE.
+#' proportion of the _full_ confidence interval range (assuming a normally
+#' distributed, equal-tailed interval) that is inside the ROPE.
+#'
+#' Note that the assumed interval, which is used to calculate the SGPV, is an
+#' _approximation_ of the _full interval_ based on the chosen confidence level.
+#' For example, if the 95% confidence interval of a coefficient ranges from -1
+#' to 1, the underlying _full (normally distributed) interval_ approximately
+#' ranges from -1.9 to 1.9, see also following code:
+#'
+#' ```
+#' # simulate full normal distribution
+#' out <- bayestestR::distribution_normal(10000, 0, 0.5)
+#' # range of "full" distribution
+#' range(out)
+#' # range of 95% CI
+#' round(quantile(out, probs = c(0.025, 0.975)), 2)
+#' ```
+#'
+#' This ensures that the SGPV always refers to the general compatible parameter
+#' space of coefficients, independent from the confidence interval chosen for
+#' testing practical equivalence. Therefore, the SGPV of the _full interval_ is
+#' similar to the ROPE coverage of Bayesian equivalence tests, see following
+#' code:
+#'
+#' ```
+#' library(bayestestR)
+#' library(brms)
+#' m <- lm(mpg ~ gear + wt + cyl + hp, data = mtcars)
+#' m2 <- brm(mpg ~ gear + wt + cyl + hp, data = mtcars)
+#' # SGPV for frequentist models
+#' equivalence_test(m)
+#' # similar to ROPE coverage of Bayesian models
+#' equivalence_test(m2)
+#' # similar to ROPE coverage of simulated draws / bootstrap samples
+#' equivalence_test(simulate_model(m))
+#' ```
 #'
 #' ## ROPE range
 #' Some attention is required for finding suitable values for the ROPE limits
 #' (argument `range`). See 'Details' in [bayestestR::rope_range()]
 #' for further information.
 #'
+#' @inheritSection model_parameters Statistical inference - how to quantify evidence
+#'
 #' @note There is also a [`plot()`-method](https://easystats.github.io/see/articles/parameters.html)
 #' implemented in the [**see**-package](https://easystats.github.io/see/).
 #'
@@ -318,7 +359,7 @@ equivalence_test.ggeffects <- function(x,
   rule <- match.arg(tolower(rule), choices = c("bayes", "classic", "cet"))
   range <- .check_rope_range(x, range, verbose)
 
-  out <- ggeffects::hypothesis_test(
+  out <- ggeffects::test_predictions(
     x,
     terms = focal,
     test = test,
@@ -335,6 +376,7 @@ equivalence_test.ggeffects <- function(x,
   l <- Map(
     function(ci_wide, ci_narrow) {
       .equivalence_test_numeric(
+        ci = ci,
         ci_wide,
         ci_narrow,
         range_rope = range,
@@ -428,11 +470,11 @@ equivalence_test.ggeffects <- function(x,
   l <- Map(
     function(ci_wide, ci_narrow) {
       .equivalence_test_numeric(
+        ci = ci,
         ci_wide,
         ci_narrow,
         range_rope = range,
         rule = rule,
-        ci = ci,
         verbose = verbose
       )
     }, conf_int, conf_int2
@@ -493,7 +535,7 @@ equivalence_test.ggeffects <- function(x,
 
   out <- do.call(rbind, lapply(names(params), function(np) {
     est <- params[[np]][, "(Intercept)"]
-    stderr <- se[[np]][, "(Intercept)"]
+    std_err <- se[[np]][, "(Intercept)"]
 
     d <- data.frame(
       Parameter = rownames(params[[np]]),
@@ -504,23 +546,23 @@ equivalence_test.ggeffects <- function(x,
     )
 
     conf_int <- as.data.frame(t(data.frame(
-      CI_low = est - stderr * fac,
-      CI_high = est + stderr * fac
+      CI_low = est - std_err * fac,
+      CI_high = est + std_err * fac
     )))
 
     conf_int2 <- as.data.frame(t(data.frame(
-      CI_low = est - stderr * fac_narrow,
-      CI_high = est + stderr * fac_narrow
+      CI_low = est - std_err * fac_narrow,
+      CI_high = est + std_err * fac_narrow
     )))
 
     l <- Map(
       function(ci_wide, ci_narrow) {
         .equivalence_test_numeric(
+          ci = ci,
           ci_wide,
           ci_narrow,
           range_rope = range,
           rule = rule,
-          ci = ci,
           verbose = verbose
         )
       }, conf_int, conf_int2
@@ -538,7 +580,12 @@ equivalence_test.ggeffects <- function(x,
 
 
 #' @keywords internal
-.equivalence_test_numeric <- function(ci_wide, ci_narrow, range_rope, rule, ci = 0.95, verbose) {
+.equivalence_test_numeric <- function(ci = 0.95,
+                                      ci_wide,
+                                      ci_narrow,
+                                      range_rope,
+                                      rule,
+                                      verbose) {
   final_ci <- NULL
 
   # ==== HDI+ROPE decision rule, by Kruschke ====
@@ -559,22 +606,14 @@ equivalence_test.ggeffects <- function(x,
 
   if (rule == "classic") {
     final_ci <- ci_narrow
-    # significant result?
-    if (min(ci_narrow) > 0 || max(ci_narrow) < 0) {
-      # check if CI are entirely inside ROPE. If CI crosses ROPE, reject H0, else accept
-      if (min(abs(ci_narrow)) < max(abs(range_rope)) && max(abs(ci_narrow)) < max(abs(range_rope))) {
-        decision <- "Accepted"
-      } else {
-        decision <- "Rejected"
-      }
-      # non-significant results
+    if (all(ci_narrow < max(range_rope)) && all(ci_narrow > min(range_rope))) {
+      # narrow CI is fully inside ROPE - always accept
+      decision <- "Accepted"
+    } else if (min(ci_narrow) < 0 && max(ci_narrow) > 0) {
+      # non-significant results - undecided
+      decision <- "Undecided"
     } else {
-      # check if CI are entirely inside ROPE. If CI crosses ROPE, reject H0, else accept
-      if (min(abs(ci_narrow)) < max(abs(range_rope)) && max(abs(ci_narrow)) < max(abs(range_rope))) {
-        decision <- "Accepted"
-      } else {
-        decision <- "Undecided"
-      }
+      decision <- "Rejected"
     }
   }
 
@@ -587,7 +626,7 @@ equivalence_test.ggeffects <- function(x,
     if (min(ci_wide) > 0 || max(ci_wide) < 0) {
       decision <- "Rejected"
       # non-significant results, all narrow CI inside ROPE
-    } else if (min(abs(ci_narrow)) < max(abs(range_rope)) && max(abs(ci_narrow)) < max(abs(range_rope))) {
+    } else if (all(ci_narrow < max(range_rope)) && all(ci_narrow > min(range_rope))) {
       decision <- "Accepted"
     } else {
       decision <- "Undecided"
@@ -597,10 +636,9 @@ equivalence_test.ggeffects <- function(x,
   data.frame(
     CI_low = final_ci[1],
     CI_high = final_ci[2],
-    SGPV = .sgpv(range_rope, final_ci),
+    SGPV = .rope_coverage(ci = ci, range_rope, ci_range = final_ci),
     ROPE_low = range_rope[1],
     ROPE_high = range_rope[2],
-    # ROPE_Percentage = .rope_coverage(range_rope, final_ci),
     ROPE_Equivalence = decision,
     stringsAsFactors = FALSE
   )
@@ -611,31 +649,34 @@ equivalence_test.ggeffects <- function(x,
 
 # helper ---------------------
 
-
-.sgpv <- function(rope, ci) {
-  diff_rope <- abs(diff(rope))
+# this function simply takes the length of the range and calculates the proportion
+# of that range that is inside the rope. However, this assumed a "flat", i.e.
+# uniformly distributed interval, which is not accurate for standard confidence
+# intervals. thus, we no longer use this function, but switch to ".rope_coverage()".
+.sgpv <- function(range_rope, ci) {
+  diff_rope <- abs(diff(range_rope))
   diff_ci <- abs(diff(ci))
 
   # inside?
-  if (min(ci) >= min(rope) && max(ci) <= max(rope)) {
+  if (min(ci) >= min(range_rope) && max(ci) <= max(range_rope)) {
     coverage <- 1
 
     # outside?
-  } else if (max(ci) < min(rope) || min(ci) > max(rope)) {
+  } else if (max(ci) < min(range_rope) || min(ci) > max(range_rope)) {
     coverage <- 0
 
     # CI covers completely rope?
-  } else if (max(ci) > max(rope) && min(ci) < min(rope)) {
+  } else if (max(ci) > max(range_rope) && min(ci) < min(range_rope)) {
     coverage <- diff_rope / diff_ci
 
     # CI inside rope and outside max rope?
-  } else if (min(ci) >= min(rope) && max(ci) > max(rope)) {
-    diff_in_rope <- max(rope) - min(ci)
+  } else if (min(ci) >= min(range_rope) && max(ci) > max(range_rope)) {
+    diff_in_rope <- max(range_rope) - min(ci)
     coverage <- diff_in_rope / diff_ci
 
     # CI inside rope and outside min rope?
-  } else if (max(ci) <= max(rope) && min(ci) < min(rope)) {
-    diff_in_rope <- max(ci) - min(rope)
+  } else if (max(ci) <= max(range_rope) && min(ci) < min(range_rope)) {
+    diff_in_rope <- max(ci) - min(range_rope)
     coverage <- diff_in_rope / diff_ci
   }
 
@@ -643,27 +684,77 @@ equivalence_test.ggeffects <- function(x,
 }
 
 
-## FIXME make sure this works for different CI levels
-.rope_coverage <- function(rope, ci_range, ci) {
-  diff_ci <- abs(diff(ci_range))
-  out <- bayestestR::distribution_normal(
-    n = 1000,
-    mean = ci_range[2] - (diff_ci / 2),
-    sd = diff_ci / 3.28
-  )
-
-  rc <- bayestestR::rope(out, range = rope, ci = ci)
+# this function simulates a normal distribution, which approximately has the
+# same range / limits as the confidence interval, thus indeed representing a
+# normally distributed confidence interval. We then calculate the probability
+# mass of this interval that is inside the ROPE.
+.rope_coverage <- function(ci = 0.95, range_rope, ci_range) {
+  out <- .generate_posterior_from_ci(ci, ci_range)
+  # compare: ci_range and range(out)
+  # The SGPV refers to the proportion of the confidence interval inside the
+  # full ROPE - thus, we set ci = 1 here
+  rc <- bayestestR::rope(out, range = range_rope, ci = 1)
   rc$ROPE_Percentage
 }
 
 
+.generate_posterior_from_ci <- function(ci = 0.95, ci_range, precision = 10000) {
+  # this function creates an approximate normal distribution that covers
+  # the CI-range, i.e. we "simulate" a posterior distribution of a
+  # frequentist CI
+  diff_ci <- abs(diff(ci_range))
+  bayestestR::distribution_normal(
+    n = precision,
+    mean = ci_range[2] - (diff_ci / 2),
+    # we divide the complete range by 2, the one-directional range for the SD.
+    # then, the range from mean value to lower/upper limit, for a normal
+    # distribution is approximately 3.3 SD (3 SD cover 99.7% of the probability
+    # mass of the normal distribution, `1 - ((1 - pnorm(3)) * 2)`). Thus,
+    # assuming that half of the ci_range refers to ~ 3.3 SD, we "normalize" the
+    # value (i.e. divide by 3.3) to get the value for one SD, which we need
+    # to build the normal distribution. The SD itself varies by confidence level,
+    # therefore we have a multiplier based on the confidence level. I agree this
+    # looks *very* hacky, but it is tested against following code, which used
+    # this code to create a normal distribution with "full" coverage, based on
+    # the approximation of the SD related to the CI-level. From this normal-
+    # distribution, the CI-level % interval is drawn and the range of the
+    # simulated normal distribution equals the desired range.
+    # -------------------------------------------------------------------------
+    # m <- lm(mpg ~ gear + hp + wt + cyl + am, data = mtcars)
+    # ci <- 0.75
+    # mp <- model_parameters(m, ci = ci)
+    # ci_range <- c(mp$CI_low[2], mp$CI_high[2])
+    # diff_ci <- abs(diff(ci_range))
+    # out <- bayestestR::distribution_normal(
+    #   n = 10000,
+    #   mean = ci_range[2] - (diff_ci / 2),
+    #   sd = diff_ci / ((stats::qnorm((1+ci)/2) * (stats::qnorm(0.999975) / 2)))
+    # )
+    # # these to ranges are roughly the same
+    # ci(out, ci = ci)
+    # ci_range
+    # -------------------------------------------------------------------------
+    # furthermore, using this approximation, following three approaches yield
+    # similar results:
+    # -------------------------------------------------------------------------
+    # m <- lm(mpg ~ gear + wt + cyl + hp, data = mtcars)
+    # m2 <- brm(mpg ~ gear + wt + cyl + hp, data = mtcars)
+    # p_significance(m, threshold = 0.6) # the default for "mpg" as response
+    # p_significance(m2)
+    # p_significance(simulate_model(m))
+    # -------------------------------------------------------------------------
+    sd = diff_ci / ((stats::qnorm((1 + ci) / 2) * (stats::qnorm(0.999975) / 2)))
+  )
+}
+
+
 .add_p_to_equitest <- function(model, ci, range) {
   tryCatch(
     {
       params <- insight::get_parameters(model)
 
       # degrees of freedom
-      df <- degrees_of_freedom(model, method = "any")
+      dof <- insight::get_df(x = model, type = "wald")
 
       # mu
       params$mu <- params$Estimate * -1
@@ -671,8 +762,8 @@ equivalence_test.ggeffects <- function(x,
       # se
       se <- standard_error(model)
 
-      stats::pt((range[1] - params$mu) / se$SE, df, lower.tail = TRUE) +
-        stats::pt((range[2] - params$mu) / se$SE, df, lower.tail = FALSE)
+      stats::pt((range[1] - params$mu) / se$SE, df = dof, lower.tail = TRUE) +
+        stats::pt((range[2] - params$mu) / se$SE, df = dof, lower.tail = FALSE)
     },
     error = function(e) {
       NULL
@@ -743,14 +834,12 @@ print.equivalence_test_lm <- function(x,
   orig_x <- x
 
   rule <- attributes(x)$rule
-  if (!is.null(rule)) {
-    if (rule == "cet") {
-      insight::print_color("# Conditional Equivalence Testing\n\n", "blue")
-    } else if (rule == "classic") {
-      insight::print_color("# TOST-test for Practical Equivalence\n\n", "blue")
-    } else {
-      insight::print_color("# Test for Practical Equivalence\n\n", "blue")
-    }
+  if (is.null(rule)) {
+    insight::print_color("# Test for Practical Equivalence\n\n", "blue")
+  } else if (rule == "cet") {
+    insight::print_color("# Conditional Equivalence Testing\n\n", "blue")
+  } else if (rule == "classic") {
+    insight::print_color("# TOST-test for Practical Equivalence\n\n", "blue")
   } else {
     insight::print_color("# Test for Practical Equivalence\n\n", "blue")
   }
@@ -788,55 +877,3 @@ plot.equivalence_test_lm <- function(x, ...) {
   insight::check_if_installed("see")
   NextMethod()
 }
-#'
-#'
-#' #' @export
-#' print_md.equivalence_test_lm <- function(x,
-#'                                          digits = 2,
-#'                                          ci_digits = digits,
-#'                                          p_digits = 3,
-#'                                          ci_brackets = NULL,
-#'                                          zap_small = FALSE,
-#'                                          ...) {
-#'   orig_x <- x
-#'
-#'   rule <- attributes(x)$rule
-#'   if (!is.null(rule)) {
-#'     if (rule == "cet") {
-#'       title <- "Conditional Equivalence Testing"
-#'     } else if (rule == "classic") {
-#'       title <- "TOST-test for Practical Equivalence"
-#'     } else {
-#'       title <- "Test for Practical Equivalence"
-#'     }
-#'   } else {
-#'     title <- "Test for Practical Equivalence"
-#'   }
-#'
-#'   .rope <- attr(x, "rope", exact = TRUE)
-#'   subtitle <- sprintf("  ROPE: [%.*f %.*f]\n\n", digits, .rope[1], digits, .rope[2])
-#'
-#'   # formatting
-#'   x <- format(x,
-#'               digits = digits,
-#'               ci_digits = ci_digits,
-#'               p_digits = p_digits,
-#'               ci_width = NULL,
-#'               ci_brackets = ci_brackets,
-#'               format = "md",
-#'               zap_small = zap_small,
-#'               ...)
-#'
-#'   if ("Group" %in% colnames(x)) {
-#'     group_by <- "Group"
-#'   } else {
-#'     group_by <- NULL
-#'   }
-#'
-#'   cat(insight::export_table(x,
-#'                             format = "md",
-#'                             title = title,
-#'                             subtitle = subtitle,
-#'                             group_by = group_by))
-#'   invisible(orig_x)
-#' }
