@@ -53,7 +53,10 @@
     # here we either have "<br>" or " " as line breaks, followed by empty "()"
     i <- gsub("<br>()", "", i, fixed = TRUE)
     i <- gsub(" ()", "", i, fixed = TRUE)
+    i <- gsub("<br>(, )", "", i, fixed = TRUE)
+    i <- gsub(" (, )", "", i, fixed = TRUE)
     i[i == "()"] <- ""
+    i[i == "(, )"] <- ""
     # remove other non-matched patterns
     i <- gsub("{stars}", "", i, fixed = TRUE)
     i <- gsub("{rhat}", "", i, fixed = TRUE)
@@ -292,15 +295,15 @@
   }
 
   # fix coefficient column name for random effects
-  if (!is.null(x$Effects) && all(x$Effects == "random") && any(colnames(x) %in% .all_coefficient_types())) {
-    colnames(x)[colnames(x) %in% .all_coefficient_types()] <- "Coefficient"
+  if (!is.null(x$Effects) && all(x$Effects == "random") && any(colnames(x) %in% .all_coefficient_types)) {
+    colnames(x)[colnames(x) %in% .all_coefficient_types] <- "Coefficient"
   }
 
   # fix coefficient column name for mixed count and zi pars
   if (!is.null(x$Component) &&
     sum(c("conditional", "zero_inflated", "dispersion") %in% x$Component) >= 2 &&
-    any(colnames(x) %in% .all_coefficient_types())) {
-    colnames(x)[colnames(x) %in% .all_coefficient_types()] <- "Coefficient"
+    any(colnames(x) %in% .all_coefficient_types)) {
+    colnames(x)[colnames(x) %in% .all_coefficient_types] <- "Coefficient"
   }
 
   # random pars with level? combine into parameter column
@@ -372,7 +375,7 @@
   }
 
   # find factors and factor levels and check if we have any factors in the data
-  factors <- .find_factor_levels(model_data)
+  factors <- .find_factor_levels(model_data, model, model_call = attributes(params)$model_call)
   if (!length(factors)) {
     params
   }
@@ -391,6 +394,17 @@
   # copy object, so we save original data
   out <- params
 
+  # sanity check - is pretty_names NULL? If so, use Parameters as pretty_names
+  if (is.null(pretty_names)) {
+    pretty_names <- stats::setNames(params$Parameter, params$Parameter)
+  }
+
+  # if we use "include_reference" and set "pretty_names = FALSE", pretty_names
+  # is no named vector. So we need to make sure we have a named vector
+  if (is.null(names(pretty_names))) {
+    pretty_names <- stats::setNames(pretty_names, params$Parameter)
+  }
+
   # if we use "keep" or "drop", we have less parameters in our data frame,
   # so we need to make sure we only have those pretty_names, which names match
   # the parameters in the data frame
@@ -406,8 +420,16 @@
     if (length(found)) {
       # the reference level is *not* in the pretty names yet
       reference_level <- f[!f %in% names(pretty_names)]
+
+      # for on-the-fly conversion of factors, the names of the factors can
+      # can also contain "factor()" or "as.factor()" - we need to remove these
+      if (any(grepl("(as\\.factor|factor|as\\.character)", fn))) {
+        fn_clean <- gsub("(as\\.factor|factor|as\\.character)\\((.*)\\)", "\\2", fn)
+      } else {
+        fn_clean <- fn
+      }
       # create a pretty level for the reference category
-      pretty_level <- paste0(fn, " [", sub(fn, "", reference_level, fixed = TRUE), "]")
+      pretty_level <- paste0(fn_clean, " [", sub(fn, "", reference_level, fixed = TRUE), "]")
       # insert new pretty level at the correct position in "pretty_names"
       pretty_names <- .insert_element_at(
         pretty_names,
@@ -432,6 +454,8 @@
   }
   # update pretty_names attribute
   attr(out, "pretty_names") <- pretty_names
+  # update pretty_labels attribute
+  pretty_names[match(names(attr(out, "pretty_labels")), names(pretty_names))] <- attr(out, "pretty_labels")
   attr(out, "pretty_labels") <- pretty_names
 
   out
@@ -444,13 +468,14 @@
 # components into different tables, we change the column name for those "tables"
 # that contain the random effects or zero-inflation parameters
 
-.all_coefficient_types <- function() {
-  c(
-    "Odds Ratio", "Risk Ratio", "Prevalence Ratio", "IRR", "Log-Odds",
-    "Log-Mean", "Log-Ratio", "Log-Prevalence", "Probability", "Marginal Means",
-    "Estimated Counts", "Ratio"
-  )
-}
+.all_coefficient_types <- c(
+  "Odds Ratio", "Risk Ratio", "Prevalence Ratio", "IRR", "Log-Odds",
+  "Log-Mean", "Log-Ratio", "Log-Prevalence", "Probability", "Marginal Means",
+  "Estimated Counts", "Ratio"
+)
+
+
+.all_coefficient_names <- c("Coefficient", "Std_Coefficient", "Estimate", "Median", "Mean", "MAP")
 
 
 .format_stan_parameters <- function(out) {
@@ -507,7 +532,7 @@
                                            is_zero_inflated,
                                            is_ordinal_model,
                                            is_multivariate = FALSE,
-                                           ran_pars,
+                                           ran_pars, # nolint
                                            formatted_table = NULL) {
   # prepare component names
   .conditional_fixed_text <- if (is_zero_inflated) {
@@ -930,16 +955,16 @@
   }
 
   # fix column output
-  if (inherits(attributes(x)$model, c("lavaan", "blavaan")) && "Label" %in% colnames(x)) {
-    x$From <- ifelse(!nzchar(as.character(x$Label), keepNA = TRUE) | x$Label == x$To, x$From, paste0(x$From, " (", x$Label, ")"))
+  if (inherits(attributes(x)[["model"]], c("lavaan", "blavaan")) && "Label" %in% colnames(x)) {
+    x$From <- ifelse(!nzchar(as.character(x$Label), keepNA = TRUE) | x$Label == x$To, x$From, paste0(x$From, " (", x$Label, ")")) # nolint
     x$Label <- NULL
   }
 
-  if (inherits(attributes(x)$model, c("lavaan", "blavaan")) && !"Parameter" %in% colnames(x)) {
+  if (inherits(attributes(x)[["model"]], c("lavaan", "blavaan")) && !"Parameter" %in% colnames(x)) {
     parameter_column <- colnames(x)[1]
   }
 
-  if (inherits(attributes(x)$model, c("lavaan", "blavaan")) && "Defined" %in% x$Component) {
+  if (inherits(attributes(x)[["model"]], c("lavaan", "blavaan")) && "Defined" %in% x$Component) {
     x$From[x$Component == "Defined"] <- ""
     x$Operator[x$Component == "Defined"] <- ""
     x$To <- ifelse(x$Component == "Defined", paste0("(", x$To, ")"), x$To)
@@ -1049,7 +1074,7 @@
     # rename columns for zero-inflation part
     if (startsWith(type, "zero") && !is.null(zi_coef_name) && !is.null(coef_column)) {
       colnames(tables[[type]])[which(colnames(tables[[type]]) == coef_column)] <- zi_coef_name
-      colnames(tables[[type]])[which(colnames(tables[[type]]) == paste0("Std_", coef_column))] <- paste0("Std_", zi_coef_name)
+      colnames(tables[[type]])[which(colnames(tables[[type]]) == paste0("Std_", coef_column))] <- paste0("Std_", zi_coef_name) # nolint
     }
 
     # rename columns for correlation, location or scale part
@@ -1063,8 +1088,8 @@
     }
 
     # rename columns for random part
-    if (grepl("random", type, fixed = TRUE) && any(colnames(tables[[type]]) %in% .all_coefficient_types())) {
-      colnames(tables[[type]])[colnames(tables[[type]]) %in% .all_coefficient_types()] <- "Coefficient"
+    if (grepl("random", type, fixed = TRUE) && any(colnames(tables[[type]]) %in% .all_coefficient_types)) {
+      colnames(tables[[type]])[colnames(tables[[type]]) %in% .all_coefficient_types] <- "Coefficient"
     }
 
     if (grepl("random", type, fixed = TRUE) && isTRUE(ran_pars)) {
@@ -1129,8 +1154,8 @@
       }
       # replace brackets by parenthesis
       if (!is.null(parameter_column) && parameter_column %in% colnames(formatted_table)) {
-        formatted_table[[parameter_column]] <- gsub("[", ci_brackets[1], formatted_table[[parameter_column]], fixed = TRUE)
-        formatted_table[[parameter_column]] <- gsub("]", ci_brackets[2], formatted_table[[parameter_column]], fixed = TRUE)
+        formatted_table[[parameter_column]] <- gsub("[", ci_brackets[1], formatted_table[[parameter_column]], fixed = TRUE) # nolint
+        formatted_table[[parameter_column]] <- gsub("]", ci_brackets[2], formatted_table[[parameter_column]], fixed = TRUE) # nolint
       }
     }
 
@@ -1151,7 +1176,7 @@
     # fix non-equal length of columns
     final_table <- .fix_nonmatching_columns(
       final_table,
-      is_lavaan = inherits(attributes(x)$model, c("lavaan", "blavaan"))
+      is_lavaan = inherits(attributes(x)[["model"]], c("lavaan", "blavaan"))
     )
     do.call(rbind, final_table)
   } else {
